//
//--------------------------------
//  Library MotorStepper
//--------------------------------
//
#include "DefinitionSystem.h"
//
#if defined(DISPATCHER_MOTORSTEPPER)
//
#ifndef MotorStepper_h
#define MotorStepper_h
//
#include "TimeSpan.h"
//
//--------------------------------
//  MotorStepper - Constant
//--------------------------------
const int MOTORSTEPPER_SIZE_ID                   = 6;
const int MOTORSTEPPER_SIZE_BUFFER               = 48;
//
const char* const MOTORSTEPPER_EVENT_MASK        = "MotorStepper[%s]=%s";
//
const char* const MOTORSTEPPER_EVENT_UNDEFINED   = "Undefined";
const char* const MOTORSTEPPER_EVENT_IDLE        = "Idle";
const char* const MOTORSTEPPER_EVENT_MOVE        = "Move";
const char* const MOTORSTEPPER_EVENT_WAIT        = "Wait";
//
const char* const STATE_MOTORSTEPPER[] = {MOTORSTEPPER_EVENT_UNDEFINED, 
                                          MOTORSTEPPER_EVENT_IDLE, 
                                          MOTORSTEPPER_EVENT_MOVE,
                                          MOTORSTEPPER_EVENT_WAIT};
//
const Int32 INIT_MOTORSTEPPER_STEP = 0;         // [stp]
const Float32 INIT_MOTORSTEPPER_WAIT = 0.0;  // [ms]
//
const Int32 INIT_MOTORSTEPPER_RANGELOW = -4000;  // [stp]
const Int32 INIT_MOTORSTEPPER_RANGEHIGH = +4000;  // [stp]
const Int32 INIT_MOTORSTEPPER_RANGEDELTA =  +100;  // [ms]
//
//--------------------------------
//  MotorStepper - Type
//--------------------------------
class CMotorStepper;
typedef void (*DOnEventMotorStepper)(CMotorStepper* plaserscanner, const char* pevent);
//
enum EStateMotorStepper
{
  ssmUndefined = 0,
  ssmIdle = 1,
  ssmMove = 2,
  ssmWait = 3  
};
//
class CMotorStepper
{
  private:
  EStateMotorStepper FState;
  char FID[MOTORSTEPPER_SIZE_ID];
  char FBuffer[MOTORSTEPPER_SIZE_BUFFER];
  CTimeSpan* FPWait;
  DOnEventMotorStepper FOnEvent;
  Int32 FStep;
  Int32 FStepLow, FStepHigh, FStepDelta;
  Float32 FWait;
  //  
  public:
  // Constructor
  CMotorStepper(const char* pid, 
                DOnEventMotorStepper oneventscanner,
                DOnEventTimeSpan oneventdelaymotion);
  // Property
  const char* StateText(EStateMotorStepper state);
  EStateMotorStepper GetState(void);
  protected:
  void SetState(EStateMotorStepper state);
  void SetStateMove(EStateMotorStepper state);
  void SetStateWait(EStateMotorStepper state);
  public:
  Int32 GetStep(void);
  void SetStep(Int32 value);
  Float32 GetWait(void);
  void SetWait(Float32 value);
  Int32 GetStepLow(void);
  void SetStepLow(Int32 value);
  Int32 GetStepHigh(void);
  void SetStepHigh(Int32 value);
  Int32 GetStepDelta(void);
  void SetStepDelta(Int32 value);
  Float32 GetDelayMotion(void);
  void SetDelayMotion(Float32 value);
  void GetRange(Int32 &steplow, 
                Int32 &stephigh, 
                Int32 &stepdelta);
  void SetRange(Int32 steplow, 
                Int32 stephigh, 
                Int32 stepdelta);
  // Method
  Boolean Open(void);
  Boolean Close(void);
  //
  Boolean Wait(Float32 wait);
  Boolean Move(Int32 step);
  Boolean MoveWait(Int32 step, Float32 wait);
  //
  Boolean Scan(void);   // SCX(XL, XH, DX, DM)
  //
  void Execute(void);
};
//
#endif // MotorStepper_h
//
#endif // DISPATCHER_MOTORSTEPPER
//
