//
#include "DefinitionSystem.h"
//
#if defined(DISPATCHER_MOTORSTEPPER)
//
// #if defined(INTERFACE_UART)
// #include "InterfaceUart.h"
// #endif
// #if defined(INTERFACE_BT)
// #include "InterfaceBt.h"
// #endif
// #if defined(INTERFACE_WLAN)
// #include "InterfaceWlan.h"
// #endif
// #if defined(INTERFACE_LAN)
// #include "InterfaceLan.h"
// #endif
//
#include "Error.h"
#include "MotorStepper.h"
#include "DispatcherMotorStepper.h"
// #if (defined(DACX_INTERNAL) | defined(DACY_INTERNAL))
// #include "DacInternal.h"
// #endif
// #if (defined(DACX_MCP4725) | defined(DACY_MCP4725))
// #include "DacMCP4725.h"
// #endif
// #if defined(DISPATCHER_TRIGGER)
// #include "TimeSpan.h"
// #endif
//
extern CError Error;
extern CCommand Command;
// 
#if defined(DISPATCHER_MOTORSTEPPER)
extern CMotorStepper MotorStepper;
#endif
//
//#########################################################
//  Dispatcher - MotorStepper - Constructor
//#########################################################
CDispatcherMotorStepper::CDispatcherMotorStepper(void)
{
}
//
//#########################################################
//  Dispatcher - MotorStepper - Execution
//#########################################################
bool CDispatcherMotorStepper::ExecuteGetStep(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: -
  // Response:
  Int32 ST = MotorStepper.GetStep();
  sprintf(Command.GetBuffer(), "%s %u", command, ST);
  ExecuteResponse(Command.GetBuffer());
  // Action: -
  ExecuteEnd();
  return true;
}
//
bool CDispatcherMotorStepper::ExecuteSetStep(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: <x>
  if (parametercount < 1)
  {
    Error.SetCode(ecMissingParameter);
    return false;
  }
  UInt32 ST = atol(parameters[0]);
  // Response:
  sprintf(Command.GetBuffer(), "%s %u", command, ST);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.SetStep(ST);
  ExecuteEnd();
  return true;
}
//
bool CDispatcherMotorStepper::ExecuteGetStepRange(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: -
  // Response:
  Int32 SL, SH, DS;
  MotorStepper.GetRange(SL, SH, DS);
  sprintf(Command.GetBuffer(), "%s %u %u %u", command, SL, SH, DS);
  ExecuteResponse(Command.GetBuffer());
  // Action: -
  ExecuteEnd();
  return true;
}
bool CDispatcherMotorStepper::ExecuteSetStepRange(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: <sl> <sh> <ds>
  if (parametercount < 3)
  {
    Error.SetCode(ecMissingParameter);
    return false;
  }
  UInt32 SL = atol(parameters[0]);
  UInt32 SH = atol(parameters[1]);
  UInt32 DS = atol(parameters[2]);
  // Response:
  sprintf(Command.GetBuffer(), "%s %u %u %u", command, SL, SH, DS);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.SetRange(SL, SH, DS);
  ExecuteEnd();
  return true;
}
//
bool CDispatcherMotorStepper::ExecuteGetStepWait(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: -
  // Response:
  Float32 DM = MotorStepper.GetDelayMotion();
  sprintf(Command.GetBuffer(), "%s %f", command, DM);
  ExecuteResponse(Command.GetBuffer());
  // Action: -
  ExecuteEnd();
  return true;
}
//
bool CDispatcherMotorStepper::ExecuteSetStepWait(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: <tw>
  if (parametercount < 1)
  {
    Error.SetCode(ecMissingParameter);
    return false;
  }
  Float32 DM = atof(parameters[0]);
  // Response:
  sprintf(Command.GetBuffer(), "%s %f", command, DM);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.SetDelayMotion(DM);
  ExecuteEnd();
  return true;
}
//
bool CDispatcherMotorStepper::ExecuteMoveStep(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: <st>
  Int32 ST = MotorStepper.GetStep();
  if (1 == parametercount)
  {
    ST = atol(parameters[0]);
  } 
  // Response:
  sprintf(Command.GetBuffer(), "%s %i", command, ST);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.Move(ST);
  ExecuteEnd();
  return true;
}
bool CDispatcherMotorStepper::ExecuteWaitStep(char* command, int parametercount, char** parameters)
{ 
  ExecuteBegin();
  // Analyse parameters: <wt>
  Float32 WT = MotorStepper.GetWait();
  if (1 == parametercount)
  {
    WT = atof(parameters[0]);
  } 
  // Response:
  sprintf(Command.GetBuffer(), "%s %f", command, WT);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.Wait(WT);
  ExecuteEnd();
  return true;
}
bool CDispatcherMotorStepper::ExecuteMoveWaitStep(char* command, int parametercount, char** parameters)
{
  ExecuteBegin();
  // Analyse parameters: <st> <wt>
  Int32 ST = MotorStepper.GetStep();
  Float32 WT = MotorStepper.GetWait();
  if (2 == parametercount)
  {
    ST = atol(parameters[0]);
    WT = atof(parameters[1]);
  } 
  else
    if (1 == parametercount)
    {
      ST = atol(parameters[0]);
    } 
  // Response:
  sprintf(Command.GetBuffer(), "%s %i %f", command, ST, WT);
  ExecuteResponse(Command.GetBuffer());
  // Action:
  MotorStepper.MoveWait(ST, WT);
  ExecuteEnd();
  return true;
}
// //
// bool CDispatcherMotorStepper::ExecuteScanX(char* command, int parametercount, char** parameters)
// {
//   ExecuteBegin();
//   // Analyse parameters: <-> 
//   // Response:
//   UInt32 XL = MotorStepperXY.GetStepXL();
//   UInt32 XH = MotorStepperXY.GetStepXH();
//   UInt32 DX = MotorStepperXY.GetStepDX();
//   Float32 DM = MotorStepperXY.GetDelayMotion();
//   sprintf(Command.GetBuffer(), "%s %u %u %u %f", command, XL, XH, DX, DM);
//   ExecuteResponse(Command.GetBuffer());
//   // Action:
//   MotorStepperXY.ScanX();
//   ExecuteEnd();
//   return true;
// }
// bool CDispatcherMotorStepper::ExecuteScanY(char* command, int parametercount, char** parameters)
// {
//   ExecuteBegin();
//   // Analyse parameters: <-> 
//   // Response:
//   UInt32 YL = MotorStepperXY.GetStepYL();
//   UInt32 YH = MotorStepperXY.GetStepYH();
//   UInt32 DY = MotorStepperXY.GetStepDY();
//   Float32 DM = MotorStepperXY.GetDelayMotion();
//   ExecuteResponse(Command.GetBuffer());
//   sprintf(Command.GetBuffer(), "%s %u %u %u %f", command, YL, YH, DY, DM);
//   // Action:
//   MotorStepperXY.ScanY();
//   ExecuteEnd();
//   return true;
// }
// bool CDispatcherMotorStepper::ExecuteScanXY(char* command, int parametercount, char** parameters)
// {
//   ExecuteBegin();
//   // Analyse parameters: <-> 
//   // Response:
//   UInt32 XL = MotorStepperXY.GetStepXL();
//   UInt32 XH = MotorStepperXY.GetStepXH();
//   UInt32 DX = MotorStepperXY.GetStepDX();
//   UInt32 YL = MotorStepperXY.GetStepYL();
//   UInt32 YH = MotorStepperXY.GetStepYH();
//   UInt32 DY = MotorStepperXY.GetStepDY();
//   Float32 DM = MotorStepperXY.GetDelayMotion();
//   sprintf(Command.GetBuffer(), "%s %u %u %u %f", command, XL, XH, DX, DM);
//   ExecuteResponse(Command.GetBuffer());
//   sprintf(Command.GetBuffer(), "%s %u %u %u %f", command, YL, YH, DY, DM);
//   ExecuteResponse(Command.GetBuffer());
//   // Action:
//   MotorStepperXY.ScanXY();
//   ExecuteEnd();
//   return true;
// }
//
//#########################################################
//  Dispatcher - MotorStepper - Handler
//#########################################################
bool CDispatcherMotorStepper::HandleInterface(char* command, int parametercount, char** parameters) 
{
  if (!strcmp(SHORT_GST, command))
  {
    return ExecuteGetStep(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_SST, command))
  {
    return ExecuteSetStep(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_GSR, command))
  {
    return ExecuteGetStepRange(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_SSR, command))
  {
    return ExecuteSetStepRange(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_GSW, command))
  {
    return ExecuteGetStepWait(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_SSW, command))
  {
    return ExecuteSetStepWait(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_MVS, command))
  {
    return ExecuteMoveStep(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_WTS, command))
  {
    return ExecuteWaitStep(command, parametercount, parameters);
  }
  if (!strcmp(SHORT_MWS, command))
  {
    return ExecuteMoveWaitStep(command, parametercount, parameters);
  }
  return false;
}
//
#endif // DISPATCHER_MOTORSTEPPER
//
