//
//--------------------------------
//  Library MotorStepper
//--------------------------------
//
#include "DefinitionSystem.h"
//
#if defined(DISPATCHER_MOTORSTEPPER)
//
#include "MotorStepper.h"
#include "InterfaceUart.h"
//
//-------------------------------------------------------------------------------
//  MotorStepper - Constructor
//-------------------------------------------------------------------------------
CMotorStepper::CMotorStepper(const char* pid, 
                             DOnEventMotorStepper oneventlaserscanner,
                             DOnEventTimeSpan oneventtimespan)
{
  strcpy(FID, pid);
  FOnEvent = 0;
  FOnEvent = oneventlaserscanner;
  FState = ssmUndefined;
  FPWait = new CTimeSpan("SCTS", oneventtimespan);
  //
  FStep = INIT_MOTORSTEPPER_STEP; // [stp]
  FWait = INIT_MOTORSTEPPER_WAIT;   // [ms]
  //
  FStepLow = INIT_MOTORSTEPPER_RANGELOW;   // [stp]
  FStepHigh = INIT_MOTORSTEPPER_RANGEHIGH;   // [stp]
  FStepDelta = INIT_MOTORSTEPPER_RANGEDELTA;   // [ms]  
}
//
//-------------------------------------------------------------------------------
//  MotorStepper - Property
//-------------------------------------------------------------------------------
const char* CMotorStepper::StateText(EStateMotorStepper state)
{
  if (ssmIdle == state) return MOTORSTEPPER_EVENT_IDLE;
  if (ssmMove == state) return MOTORSTEPPER_EVENT_MOVE;
  if (ssmWait == state) return MOTORSTEPPER_EVENT_WAIT;
  return MOTORSTEPPER_EVENT_UNDEFINED;
}
EStateMotorStepper CMotorStepper::GetState(void)
{
  return FState;
}
void CMotorStepper::SetState(EStateMotorStepper state)
{
  if (state != FState)
  {
    FState = state;
    if (0 != FOnEvent)
    {
      sprintf(FBuffer, "MotorStepper[%s]: %s", FID, StateText(FState));
      FOnEvent(this, FBuffer);
    }
  }
}
void CMotorStepper::SetStateMove(EStateMotorStepper state)
{
  if (state != FState)
  {
    FState = state;
    if (0 != FOnEvent)
    {
      sprintf(FBuffer, "MotorStepper[%s]: %s %u", FID, StateText(FState), FStep);
      FOnEvent(this, FBuffer);
    }
  }
}
void CMotorStepper::SetStateWait(EStateMotorStepper state)
{
  if (state != FState)
  {
    FState = state;
    if (0 != FOnEvent)
    {
      sprintf(FBuffer, "MotorStepper[%s]: %s %f", FID, StateText(FState), FWait);
      FOnEvent(this, FBuffer);
    }
  }
}

Int32 CMotorStepper::GetStep(void)
{
  return FStep;
}
void CMotorStepper::SetStep(Int32 value)
{
  FStep = value;
}
//
Float32 CMotorStepper::GetWait(void)
{
  return FWait;
}
void CMotorStepper::SetWait(Float32 value)
{
  FWait = value;
}
//
Int32 CMotorStepper::GetStepLow(void)
{
  return FStepLow;
}
void CMotorStepper::SetStepLow(Int32 value)
{
  FStepLow = value;
}
//
Int32 CMotorStepper::GetStepHigh(void)
{
  return FStepHigh;
}
void CMotorStepper::SetStepHigh(Int32 value)
{
  FStepHigh = value;
}
//
Int32 CMotorStepper::GetStepDelta(void)
{
  return FStepDelta;
}
void CMotorStepper::SetStepDelta(Int32 value)
{
  FStepDelta = value;
}
//
Float32 CMotorStepper::GetDelayMotion(void)
{
  return FWait;
}
void CMotorStepper::SetDelayMotion(Float32 value)
{
  FWait = value;
}
//
void CMotorStepper::GetRange(Int32 &positionlow, 
                             Int32 &positionhigh, 
                             Int32 &positiondelta)
{
  positionlow = FStepLow;  
  positionhigh = FStepHigh;  
  positiondelta = FStepDelta;  
}
void CMotorStepper::SetRange(Int32 positionlow,
                             Int32 positionhigh, 
                             Int32 positiondelta)
{
  FStepLow = positionlow;
  FStepHigh = positionhigh;  
  FStepDelta = positiondelta;
}
//
//-------------------------------------------------------------------------------
//  MotorStepper - Method
//-------------------------------------------------------------------------------
Boolean CMotorStepper::Open()
{
  FPWait->Open();
  //
  FStep = INIT_MOTORSTEPPER_STEP;      // [stp]
  FWait = INIT_MOTORSTEPPER_WAIT;  // [ms]
  //
  FStepLow = INIT_MOTORSTEPPER_RANGELOW;   // [stp]
  FStepHigh = INIT_MOTORSTEPPER_RANGEHIGH;   // [stp]
  FStepDelta = INIT_MOTORSTEPPER_RANGEDELTA;   // [ms]
  //
  SetState(ssmIdle);
  return true;
}
//
Boolean CMotorStepper::Close()
{
  SetState(ssmUndefined);
  return true;
}
//
Boolean CMotorStepper::Move(Int32 step)
{
  FStep = step;
  // FTimeWait - no change!
  // ... FPDacX->SetValue(FStepX);
  // ... FPDacY->SetValue(FStepY);
  SetState(ssmMove);
  return true;
}  
Boolean CMotorStepper::Wait(Float32 wait)
{
  // FStepX - no change!
  // FStepY - no change!
  FWait = wait;
  FPWait->StartPeriod(FWait);
  SetState(ssmWait);
  return true;
}  
Boolean CMotorStepper::MoveWait(Int32 step, Float32 wait)
{
  FStep = step;
  FWait = wait;
  // ... FPDacX->SetValue(FStepX);
  // ... FPDacY->SetValue(FStepY);
  FPWait->StartPeriod(FWait);
  SetState(ssmWait);
  return true;
}
//
/*Boolean CMotorStepper::ScanX(void)   // SCX(XL, XH, DX, DM)
{
  FStepX = FStepXL;
  FStepY = FStepY;
  // FDelayMotion
  FPDacX->SetValue(FStepX);
  FPDacY->SetValue(FStepY);
  SetStateScanX(ssmScanX);
  FPDelayMotion->StartPeriod(FDelayMotion);
  SetStateWaitX(ssmWaitX);
  return true;
}
Boolean CMotorStepper::ScanY(void)   // SCY(YL, YH, DY, DM)
{
  FStepX = FStepX;
  FStepY = FStepYL;
  // FDelayMotion
  FPDacX->SetValue(FStepX);
  FPDacY->SetValue(FStepY);
  SetStateScanY(ssmScanY);
  FPDelayMotion->StartPeriod(FDelayMotion);
  SetStateWaitY(ssmWaitY);
  return true;
}
Boolean CMotorStepper::ScanXY(void)  // SXY(XL, XH, DX, YL, YH, DY, DM)
{
  FStepX = FStepXL;
  FStepY = FStepYL;
  // FDelayMotion
  FPDacX->SetValue(FStepX);
  FPDacY->SetValue(FStepY);
  SetStateScanXY(ssmScanXY);
  FPDelayMotion->StartPeriod(FDelayMotion);
  SetStateWaitXY(ssmWaitXY);
  return true;
}*/
//
//-------------------------------------------------------------------------------
//  MotorStepper - Execute
//-------------------------------------------------------------------------------
void CMotorStepper::Execute(void)
{ // debug Serial.print(".");
  // debug delay(2999);
  FPWait->Execute();
  switch (FState)
  {
    case ssmUndefined:
      return;
    case ssmIdle:
      return;
    case ssmMove:
      // NO wait !!!FPDelayMotion->StartPeriod(FDelayMotion);
      // SetState(slsWait);
      SetState(ssmIdle);
      return;
    case ssmWait:
      if (FPWait->IsIdle())
      {
        SetState(ssmIdle);
      }
      return;
    //-------------------------------------
    // case ssmScanX:
    //   FPDacX->SetValue(FStepX);
    //   FPDacY->SetValue(FStepY);
    //   FPDelayMotion->StartPeriod(FDelayMotion);
    //   SetStateWaitX(ssmWaitX);
    //   return;
    // case ssmWaitX:
    //   if (FPDelayMotion->IsIdle())
    //   {
    //     FStepX += FStepDX;
    //     if (FStepX <= FStepXH)
    //     {
    //       SetStateScanX(ssmScanX);
    //     }
    //     else
    //     {
    //       SetState(ssmIdle);
    //     }        
    //   }
    //   return;
    // //-------------------------------------
    // case ssmScanY:
    //   FPDacX->SetValue(FStepX);
    //   FPDacY->SetValue(FStepY);
    //   FPDelayMotion->StartPeriod(FDelayMotion);
    //   SetStateWaitY(ssmWaitY);
    //   return;
    // case ssmWaitY:
    //   if (FPDelayMotion->IsIdle())
    //   {
    //     FStepY += FStepDY;
    //     if (FStepY <= FStepYH)
    //     {
    //       SetStateScanY(ssmScanY);
    //     }
    //     else
    //     {
    //       SetState(ssmIdle);
    //     }        
    //   }
    //   return;
    // //-------------------------------------
    // case ssmScanXY:
    //   FPDacX->SetValue(FStepX);
    //   FPDacY->SetValue(FStepY);
    //   FPDelayMotion->StartPeriod(FDelayMotion);
    //   SetStateWaitXY(ssmWaitXY);
    //   return;
    // case ssmWaitXY:
    //   if (FPDelayMotion->IsIdle())
    //   {
    //     FStepX += FStepDX;
    //     if (FStepX <= FStepXH)
    //     {
    //       SetStateScanXY(ssmScanXY);
    //     }
    //     else
    //     {
    //       FStepX = FStepXL;
    //       FStepY += FStepDY;
    //       if (FStepY <= FStepYH)
    //       {
    //         SetStateScanXY(ssmScanXY);
    //       }
    //       else
    //       {
    //         FStepY = FStepYL;
    //         SetState(ssmIdle);
    //       }
    //     }
    //   }
    //   return;    
    //-------------------------------------
  }
}
//
#endif // DISPATCHER_MOTORSTEPPER
//
